/*
-------------------------------------------------------------------------------
    This file is a part of the VST Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: maindialog.cpp
  Author: Francois Mazen
  Date: 28/05/07 16:25
  Description: the MainDialog class, the core...
-------------------------------------------------------------------------------
*/

#include "maindialog.h"

#include "filetypedialog.h"
#include "endiannessconverter.h"
#include "vpgversion.h"
#include "vpgfile.h"
#include "vstpreset.h"

#include <QtWidgets>

#define VPG_LOGO ":/images/vpg.png" //logo filename

//-----------------------------------------------------------------------------
//MainDialog Class
//-----------------------------------------------------------------------------

//constructor
MainDialog::MainDialog(VpgSettings &settings) : QMainWindow(),
    vpgSettings(settings)
{
    //Qt objects------------------------------------------
        //buttons:
    QAction *genAction = new QAction(QIcon(":/images/generate.png"),tr("&Generate a preset file!"), this);
    btnGen = new QPushButton(QIcon(":/images/generate.png"),tr("&Generate a preset file!"), this);
    btnGen->setIconSize (QSize(32,32));
        //label:
    headerDescription = new QLabel(tr("No header loaded"));

        //containers:
    QWidget *mainContainer = new QWidget(this);
    QWidget *bottomContainer = new QWidget(mainContainer);
    QWidget *topContainer = new QWidget(mainContainer);

        //layout:
    QHBoxLayout *mainLayout = new QHBoxLayout;
    QVBoxLayout *bottomLayout = new QVBoxLayout;
    QVBoxLayout *topLayout = new QVBoxLayout;

        //Progress Bar
    progressBar = new QProgressBar(bottomContainer);
        //List
    vstList = new QListView();
        //Image
    vpgLogo = new const QPixmap(VPG_LOGO);
    QLabel *lblLogo = new QLabel;
    lblLogo->setPixmap(*vpgLogo);

        //VST host
    vpgVstHost = new VpgVstHost();

        //table for parameters diplay
    parameterTable = new QTableView(this);
    parameterTableModel = new ParameterTableModel();
    paramValueDelegate = new ParamValueDelegate(this);
    randomModeDelegate = new RandomModeDelegate(this);
    parameterTable->setModel(parameterTableModel);
    parameterTable->setItemDelegateForColumn(1,randomModeDelegate);
    parameterTable->setItemDelegateForColumn(2,paramValueDelegate);
    parameterTable->setItemDelegateForColumn(3,paramValueDelegate);
    parameterTable->setEditTriggers(QAbstractItemView::AllEditTriggers);

        //VST list
    vstListModel = new VSTListModel(this);
    vstList->setModel(vstListModel);

        //fxHeader
    fxHeader = new FxHeader();


    //Fill layout------------------------------------------
    vstListToolBar = new QToolBar(tr("VST List"));
    topLayout->setContentsMargins(11,0,11,11);
    topLayout->addWidget(vstListToolBar);
    topLayout->addWidget(vstList);
    topLayout->addWidget(headerDescription);
    topLayout->addWidget(btnGen);
    topContainer->setLayout(topLayout);
        //bottom layout
    bottomLayout->addWidget(parameterTable);
    bottomLayout->addWidget(progressBar);
    bottomContainer->setLayout(bottomLayout);
        //global layout
    mainLayout->addWidget(bottomContainer);
    mainContainer->setLayout(mainLayout);
    setCentralWidget(mainContainer);

    //dock ----------------------------------------------
    QDockWidget *dockList = new QDockWidget(tr("VST List"), this);
    dockList->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
    dockList->setFeatures(QDockWidget::DockWidgetMovable);
    dockList->setWidget(topContainer);
    addDockWidget(Qt::LeftDockWidgetArea, dockList);


    //Menus-------------------------------------------------
        //Import
    QAction *importVst = new QAction(QIcon(":/images/import.png"),tr("Import informations about &VST Plugin"),this);
    QMenu *importMenu = menuBar()->addMenu(tr("&VST"));
    importMenu->addAction(importVst);
    importMenu->addAction(genAction);
        //VST List
    QAction *openVstListAction = new QAction(QIcon(":/images/open.png"),tr("&Open VST List..."), this);
    QAction *saveVstListAction = new QAction(QIcon(":/images/save.png"),tr("&Save VST List..."), this);
    QAction *newVstListAction = new QAction(QIcon(":/images/new.png"),tr("&New VST List"), this);
    QMenu *vstListMenu = menuBar()->addMenu(tr("&VST List"));
    vstListMenu->addAction(newVstListAction);
    vstListMenu->addAction(openVstListAction);
    vstListMenu->addAction(saveVstListAction);
        //Random modification
    QAction *setUniformRandom = new QAction(QIcon(":/images/uniform.png"),tr("Set &uniform random method to all parameters"), this);
    QAction *setNormalRandom = new QAction(QIcon(":/images/normal.png"),tr("Set &normal random method to all parameters"), this);
    QAction *setConstantRandom = new QAction(QIcon(":/images/constant.png"),tr("Set &constant values to all parameters"), this);
    QAction *setTotalRandom = new QAction(QIcon(":/images/total_random.png"),tr("Set &total random values"), this);
    QMenu *randomMenu = menuBar()->addMenu(tr("&Random methods"));
    randomMenu->addAction(setUniformRandom);
    randomMenu->addAction(setNormalRandom);
    randomMenu->addAction(setConstantRandom);
    randomMenu->addAction(setTotalRandom);
        //About
    QAction *aboutAction = new QAction(QIcon(":/images/vpg.png"),tr("&About VST Preset Generator"), this);
    QAction *aboutQtAction = new QAction(tr("About &Qt"), this);
    QMenu *aboutMenu = menuBar()->addMenu(tr("&About"));
    aboutMenu->addAction(aboutAction);
    aboutMenu->addAction(aboutQtAction);


    //Toolbars-----------------------------------------------
    vstListToolBar->addAction(newVstListAction);
    vstListToolBar->addAction(openVstListAction);
    vstListToolBar->addAction(saveVstListAction);
    vstListToolBar->setIconSize(QSize(16,16));
    vstImportToolBar = addToolBar(tr("Import VST"));
    vstImportToolBar->addAction(importVst);
    vstImportToolBar->addAction(genAction);
    randomMethodToolBar = addToolBar(tr("Random methods"));
    randomMethodToolBar->addAction(setUniformRandom);
    randomMethodToolBar->addAction(setNormalRandom);
    randomMethodToolBar->addAction(setConstantRandom);
    randomMethodToolBar->addAction(setTotalRandom);



    //Qt connexions------------------------------------------
        //buttons:
    connect(btnGen, SIGNAL(clicked()), this, SLOT(generatePreset()));
    connect(genAction, SIGNAL(triggered()), this, SLOT(generatePreset()));
        //headers:
    connect(this, SIGNAL(validHeader(FxHeaderStruct*)),
            fxHeader, SLOT(receiveParameters(FxHeaderStruct*)));
    connect(this, SIGNAL(validHeader(FxHeaderStruct*)),
            this, SLOT(showLabelHeader(FxHeaderStruct*)));
    connect(fxHeader, SIGNAL(headerIsValid(bool)),
            btnGen, SLOT(setEnabled(bool)));
    connect(fxHeader, SIGNAL(headerIsValid(bool)),
            randomMenu, SLOT(setEnabled(bool)));
    connect(fxHeader, SIGNAL(headerIsValid(bool)),
            randomMethodToolBar, SLOT(setEnabled(bool)));
    connect(fxHeader, SIGNAL(headerIsValid(bool)),
            genAction, SLOT(setEnabled(bool)));


    connect(fxHeader, SIGNAL(headerIsValid(bool)),
            this, SLOT(updateParamList(bool)));
        //list interaction
    connect(vstList, SIGNAL(doubleClicked ( const QModelIndex &)),
            this, SLOT(onVstListDoubleClicked(const QModelIndex &)));
        //menu connections
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(showCopyright()));
    connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
    connect(openVstListAction, SIGNAL(triggered()), this, SLOT(openVstList()));
    connect(saveVstListAction, SIGNAL(triggered()), this, SLOT(saveVstList()));
    connect(newVstListAction, SIGNAL(triggered()), this, SLOT(newVstList()));
    connect(importVst, SIGNAL(triggered()), this, SLOT(importParamFromVstOrPreset()));
    connect(setUniformRandom, SIGNAL(triggered()), this, SLOT(setGlobalRandomMethodUniform()));
    connect(setNormalRandom, SIGNAL(triggered()), this, SLOT(setGlobalRandomMethodNormal()));
    connect(setConstantRandom, SIGNAL(triggered()), this, SLOT(setGlobalRandomMethodConstant()));
    connect(setTotalRandom, SIGNAL(triggered()), this, SLOT(setGlobalRandomMethodTotal()));
    connect(parameterTableModel, SIGNAL(dataChanged( const QModelIndex &, const QModelIndex & )), parameterTable, SLOT(resizeColumnsToContents()));

    //init values----------------------------------------
    btnGen->setEnabled(false);


    //application settings:
    QString title(tr("VST Preset Generator V"));
    title += VpgVersion::getCurrentVersion().getStringVersionWithPlatform();
    setWindowTitle(title);
    resize(700,500);
    setWindowIcon(QIcon(":/images/vpg_icon.png"));

    FxHeaderStruct t_fxheaderstruct;
    t_fxheaderstruct.name = "";
    t_fxheaderstruct.fxID = "";
    t_fxheaderstruct.fxVersion = 0;
    t_fxheaderstruct.nParams = 0;
    t_fxheaderstruct.useChunk = false;
    fxHeader->receiveParameters(&t_fxheaderstruct);
    showLabelHeader(&t_fxheaderstruct);


    //enable to drop file on the application
    setAcceptDrops(true);
}


MainDialog::~MainDialog()
{
    delete fxHeader;
    delete vstListModel;
    delete vpgVstHost;
    delete parameterTableModel;
    delete vstListToolBar;
    delete randomMethodToolBar;
    delete vstImportToolBar;
    delete btnGen;
    delete headerDescription;
    delete vstList;
    delete vpgLogo;
    delete progressBar;
}

//-----------------------------------------------------------------------------
//Dropping functions from Qt's patterns
//-----------------------------------------------------------------------------
void MainDialog::dragEnterEvent(QDragEnterEvent *event)
{
    if (event->mimeData()->hasUrls())
    {
        event->acceptProposedAction();
    }
}
//-----------------------------------------------------------------------------
void MainDialog::dropEvent(QDropEvent *event)
{
    if(event->mimeData()->hasUrls())
    {
        QList<QUrl> urlList = event->mimeData()->urls();
        //QMessageBox::critical(this, "debug",urlList[0].path());
        //if there is many file, we only grab the first:
        QString fileName = urlList[0].path();

        if(fileName != NULL)
        {
            fileName.remove(0,1); //remove the "/" at the beginnign of the url
            if(fileName.endsWith (QString("dll")))
            {
                loadParamFromVst(fileName);
            }
            else
            {
                openPreset(fileName);
            }

            event->acceptProposedAction();
        }
    }
}
//-----------------------------------------------------------------------------

void MainDialog::openPreset(QString fileName)
{
    if(!fileName.isEmpty())
    {
        //open the file
        QFile file(fileName);
        if (!file.open(QIODevice::ReadOnly))
        {
            //error while opening
            QMessageBox::critical(this, tr("Opening Preset File"),
                                        tr("Could not read the file"));
        }
        else
        {
            QFileInfo info(file);
            this->vpgSettings.setGeneratedPresetFolder(info.absolutePath());

            QByteArray buffer = file.read(4);
            if(!buffer.startsWith("CcnK"))
            {
                //not a preset file
                QMessageBox::critical(this, tr("Opening Preset File"),
                                            tr("This is not a valid VST preset file"));
            }
            else
            {
                buffer = file.read(4);//chunk size
                buffer = file.read(4);//type of file (bank or program)

                QString t_fxID;
                qint32 t_fxVersion ;
                qint32 t_nParams ;

                //'FxCk' (regular) or 'FPCh' (opaque chunk) Program
                //'FxBk' (regular) or 'FBCh' (opaque chunk) Bank
                bool isFileValid = false;
                if(buffer.startsWith("FxCk") == true)
                {
                    //it's a program or a bank
                    buffer = file.read(4);//format version

                    buffer = file.read(4);//fx unique ID
                    t_fxID = buffer.data();

                    buffer = file.read(4);//fx version
                    t_fxVersion = EndiannessConverter::fromBigEndian(buffer.data());

                    buffer = file.read(4);//number of parameters
                    t_nParams = EndiannessConverter::fromBigEndian(buffer.data());

                    isFileValid = true;
                }
                else if(buffer.startsWith("FPCh") == true)
                {
                    //Opaque chunk data, not suported
                    QMessageBox::critical(this, tr("Loading Preset File"),
                                                tr("Opaque Chunk method : please import informations from the VST file (.dll)"));
                }
                else if(buffer.startsWith("FxBk") == true)
                {
                    //bank program

                    //jump to the first program and read the parameters
                    file.read(4*4 + 128 + 4*3);

                    buffer = file.read(4);//format version

                    buffer = file.read(4);//fx unique ID
                    t_fxID = buffer.data();

                    buffer = file.read(4);//fx version
                    t_fxVersion = EndiannessConverter::fromBigEndian(buffer.data());

                    buffer = file.read(4);//number of parameters
                    t_nParams = EndiannessConverter::fromBigEndian(buffer.data());

                    isFileValid = true;
                }
                else if(buffer.startsWith("FBCh") == true)
                {
                    //Opaque chunk data, not suported
                    QMessageBox::critical(this, tr("Loading Preset File"),
                                                tr("Opaque Chunk method : please import informations from the VST file (.dll)"));
                }
                else
                {
                    //invalid file (no program or no bank)
                    QMessageBox::critical(this, tr("Loading Preset File"),
                                                tr("This is not a valid VST preset file"));
                }

                if(isFileValid)
                {
                    //Get the name of VST from the user:
                    bool isNameOK = false;
                    QString vstName = QInputDialog::getText(this, tr("Name of the VST"),
                                                                   tr("Please enter the name of your VST:"), QLineEdit::Normal,
                                                                   tr("My VST"), &isNameOK);

                    FxHeaderStruct *fxHeader = new FxHeaderStruct;
                    fxHeader->fxID = t_fxID;
                    fxHeader->fxVersion = t_fxVersion;
                    fxHeader->nParams = t_nParams;
                    fxHeader->useChunk = false;
                    if (isNameOK && !vstName.isEmpty())
                    {
                        fxHeader->name = vstName;
                    }
                    else
                    {
                        fxHeader->name = tr("Invalid Name");
                    }

                    //add the new VST to our list:
                    vstListModel->addPluginHeader(fxHeader);

                    //send parameters to the headers object
                    emit validHeader(fxHeader);
                }
            }

            file.close();
        }
    }
}

//-----------------------------------------------------------------------------

void MainDialog::generatePreset()
{
    //get the type of the file

    int maxProgramPerBank = 100;
    bool isBankFileEnabled = true;
    if(this->fxHeader->Data.useChunk)
    {
        maxProgramPerBank = vpgVstHost->getNumberPrograms();
        if(maxProgramPerBank <= 0)
        {
            isBankFileEnabled = false;
        }
    }

    FileTypeDialog fileTypeDialog(this, isBankFileEnabled, maxProgramPerBank);
    FileTypeDialog::FILE_TYPE fileType = (FileTypeDialog::FILE_TYPE)fileTypeDialog.exec();

    if(fileType != FileTypeDialog::FILE_TYPE_INVALID) //Cancel Button
    {
        int programCount = 1;
        bool isPresetTypeBank = false;
        if(fileType == FileTypeDialog::FILE_TYPE_BANK)
        {
            isPresetTypeBank = true;
            programCount = fileTypeDialog.getProgramPerBankValue();
        }

        //get the name of file
        QString fileName;
        if(isPresetTypeBank)
        {
            fileName = QFileDialog::getSaveFileName(this,
                                              tr("Generate Preset Bank"),
                                              this->vpgSettings.getGeneratedPresetFolder(),
                                              tr("Preset Bank (*.fxb)"));
        }
        else
        {
            fileName = QFileDialog::getSaveFileName(this,
                                              tr("Generate Preset Program"),
                                              this->vpgSettings.getGeneratedPresetFolder(),
                                              tr("Preset Program (*.fxp)"));
        }

        if(fileName != NULL) //cancel button
        {
            // Generate the content of the file.
            VSTPreset::ERROR_CODE presetErrorCode = VSTPreset::ERROR_NO_ERROR;

            if(fxHeader->Data.useChunk)
            {
                QString loadedFxId = EndiannessConverter::getMultiCharFromBigEndian(vpgVstHost->getFxId());

                //Check the VST loaded
                bool isVSTLoaded = false;
                if((vpgVstHost->getFxId() == 0) ||
                   (loadedFxId != fxHeader->Data.fxID))
                {

                    QMessageBox::warning(this, tr("Generating Preset File"),
                                               tr("Please load the VST plugin file related to the selected preset."));
                    //get the name of the file
                    fileName = QFileDialog::getOpenFileName(this, tr("Load a Plugin VST"),
                                                                  fxHeader->Data.fxPath,
                                                                  tr("VST Plugin (*.dll);;Linux VST Plugin (*.so)"));
                    if(fileName != NULL && loadVst(fileName))
                    {
                        QString newLoadedFxId = EndiannessConverter::getMultiCharFromBigEndian(vpgVstHost->getFxId());

                        if(newLoadedFxId != fxHeader->Data.fxID)
                        {
                            QMessageBox::critical(this, tr("Generating Preset File"),
                                                        tr("The VST plugin file doesn't have the same ID has the selected preset."
                                                           "\nPlease try again."));
                        }
                        else
                        {
                            isVSTLoaded = true;
                        }
                    }
                }
                else
                {
                    isVSTLoaded = true;
                }

                if(isVSTLoaded)
                {
                    if(isPresetTypeBank)
                    {
                        presetErrorCode = VSTPreset::WriteBankChunkToFile(fileName, *(this->fxHeader), *(this->parameterTableModel), *(this->vpgVstHost), programCount);
                    }
                    else
                    {
                        presetErrorCode = VSTPreset::WriteProgramChunkToFile(fileName, *(this->fxHeader), *(this->parameterTableModel), *(this->vpgVstHost));
                    }
                }
                else
                {
                    presetErrorCode = VSTPreset::ERROR_COULD_NOT_CREATE_FILE;
                }
            }
            else //non-chunk generation
            {
                if(isPresetTypeBank)
                {
                    presetErrorCode = VSTPreset::WriteBankToFile(fileName, *(this->fxHeader), *(this->parameterTableModel), programCount);
                }
                else
                {
                    presetErrorCode = VSTPreset::WriteProgramToFile(fileName, *(this->fxHeader), *(this->parameterTableModel));
                }
            }

            // Display error/success messages.
            if(presetErrorCode == VSTPreset::ERROR_NO_ERROR)
            {
                QFileInfo info(fileName);
                this->vpgSettings.setGeneratedPresetFolder(info.absolutePath());

                //Operation success
                QString success(tr("The preset file:\n"));
                success.append(fileName);
                success.append(tr("\nhas been created. Enjoy!"));
                QMessageBox::information(this, tr("Saving Preset File"), success);
            }
            else if (presetErrorCode == VSTPreset::ERROR_COULD_NOT_CREATE_FILE)
            {
                QMessageBox::critical(this, tr("Saving Preset File"),
                                            tr("Could not create the file"));
            }
            else if(presetErrorCode == VSTPreset::ERROR_VST_HOST_INVALID_PROGRAM_NUMBER)
            {
                QMessageBox::critical(this, tr("Generating Preset File"),
                                            tr("Can't generate Bank file for this VST plug-in."
                                               "\nPlease try with a Program file."));
            }
        }
    }
}

//-----------------------------------------------------------------------------

void MainDialog::showLabelHeader(FxHeaderStruct* data)
{
    QString description = tr("VST name: ");
    description += data->name;

    description += tr("\nVST ID: ");
    description += data->fxID;

    description += tr("\nVST Version: ");
    QString version;
    version.setNum(data->fxVersion);
    description += version;

    description += tr("\nnumber of parameters: ");
    QString parameterCount;
    parameterCount.setNum(data->nParams);
    description += parameterCount;

    description += tr("\npreset type: ");
    if(data->useChunk)
    {
        description += tr("chunk");
    }
    else
    {
        description += tr("parameters");
    }

    headerDescription->setText(description);
}

//-----------------------------------------------------------------------------
void MainDialog::updateParamList(bool isHeaderValid)
{
    parameterTableModel->beginModelChangeWithoutRefresh();

    parameterTableModel->clearWithoutRefresh();
    parameterTableModel->setParamNbWithoutRefresh(fxHeader->Data.nParams);

    //avoid problems while using a no-chunk-plug-in
    if(fxHeader->Data.useChunk == false)
    {
        vpgVstHost->close();
    }
    else if(vpgVstHost->isVstLoaded() == false ||
            (QFileInfo(fxHeader->Data.fxPath) != QFileInfo(vpgVstHost->getLoadedFilename())))
    {
        vpgVstHost->close();
        VpgVstHost::ERROR_CODE errorCode = vpgVstHost->load(fxHeader->Data.fxPath);
        if(errorCode != VpgVstHost::ERROR_NO_ERROR)
        {
            isHeaderValid = false;
        }
    }

    if(isHeaderValid)
    {
        progressBar->setMinimum(0);
        progressBar->setMaximum(fxHeader->Data.nParams);

        for(int i =1; i<=fxHeader->Data.nParams; i++)
        {
            progressBar->setValue(i);

            //if we have loaded the plugin (and not only a preset file)
            //we get the name of each parameter
            Parameter parameter(tr("unknown"));
            if(fxHeader->Data.useChunk == true && vpgVstHost->isVstLoaded())
            {
                parameter.setName(vpgVstHost->getParameterName(i-1));
            }

            parameterTableModel->replaceParameterWithoutRefresh(i, parameter);
        }
    }

    parameterTableModel->endModelChangeWithoutRefresh();

    parameterTable->resizeColumnsToContents();
    parameterTable->resizeRowsToContents();
}

//-----------------------------------------------------------------------------
void MainDialog::saveVstList()
{
    //get the name of the file
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save VST List"),
                                                          this->vpgSettings.getVpgFilesFolder(),
                                                          tr("VST Preset Generator file (*.vpg)"));
    if(fileName != NULL)
    {
        //save the folder for future use
        QFileInfo info(fileName);
        this->vpgSettings.setVpgFilesFolder(info.absolutePath());

        VpgFile::ERROR_CODE errorCode = VpgFile::WriteFile(fileName, *vstListModel);
        if(errorCode != VpgFile::ERROR_NO_ERROR)
        {
            QString errorMessage;
            switch(errorCode)
            {
            case VpgFile::ERROR_COULD_NOT_OPEN_FILE:
                //error while opening
                errorMessage = tr("Error opening file: ");
                errorMessage.append(fileName);
                break;
            case VpgFile::ERROR_COULD_NOT_PARSE_FILE:
            case VpgFile::ERROR_INVALID_VPG_FILE:
            case VpgFile::ERROR_UNKNOWN_XML_TAG:
            case VpgFile::ERROR_INVALID_XML_ELEMENT:
            case VpgFile::ERROR_NO_ERROR:
                break;
            }

            QMessageBox::critical(this, tr("Saving VST List"), errorMessage);
        }

        vstListModel->setNeedSaving(false);
    }
}

//-----------------------------------------------------------------------------
void MainDialog::openVstList()
{

    //get the name of the file
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open VST List"),
                                                          this->vpgSettings.getVpgFilesFolder(),
                                                          tr("VST Preset Generator file (*.vpg)"));
    if(fileName != NULL)
    {
        //save the folder for future use
        QFileInfo info(fileName);
        this->vpgSettings.setVpgFilesFolder(info.absolutePath());

        VpgFile::ERROR_CODE errorCode = VpgFile::ReadFile(fileName, *vstListModel);

        if(errorCode != VpgFile::ERROR_NO_ERROR)
        {
            QString errorMessage;
            switch(errorCode)
            {
            case VpgFile::ERROR_COULD_NOT_OPEN_FILE:
                errorMessage = tr("Could not open the file: ");
                errorMessage.append(fileName);
                break;
            case VpgFile::ERROR_COULD_NOT_PARSE_FILE:
                errorMessage = tr("Could not parse the file: ");
                errorMessage.append(fileName);
                break;
            case VpgFile::ERROR_INVALID_VPG_FILE:
                errorMessage = tr("The file is not a valid vpg file: ");
                errorMessage.append(fileName);
                break;
            case VpgFile::ERROR_UNKNOWN_XML_TAG:
                errorMessage = tr("Unknown XML tag in file: ");
                errorMessage.append(fileName);
                break;
            case VpgFile::ERROR_INVALID_XML_ELEMENT:
                errorMessage = tr("Invalid XML element in file: ");
                errorMessage.append(fileName);
                break;
            case VpgFile::ERROR_NO_ERROR:
                break;
            }
            QMessageBox::information(this, tr("Opening VST List"), errorMessage);
        }

        vstListModel->setNeedSaving(false);

        //emit an empty header
        FxHeaderStruct *invalid = new FxHeaderStruct;
        invalid->name = "";
        invalid->fxID = "";
        invalid->fxVersion = 0;
        invalid->nParams = 0;

        emit validHeader(invalid);
    }
}

//-----------------------------------------------------------------------------
void MainDialog::newVstList()
{
    if(!vstListModel->isListEmpty())
    {
        bool mustClearTheList = true;
        if(!vstListModel->areDataSaved())
        {
            //ask for saving the actual list
            QMessageBox::StandardButton clickedButton = QMessageBox::question(this,
                                                                              tr("New VST List..."),
                                                                              tr("Do you want to save changes to your current VST list?"),
                                                                              QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);

            if(clickedButton == QMessageBox::Cancel)
            {
                //quit the function
                mustClearTheList = false;
            }
            else if(clickedButton == QMessageBox::Save)
            {
                saveVstList();
            }
        }

        if(mustClearTheList)
        {
            //clear the list
            vstListModel->clearList();

            //emit an empty header
            FxHeaderStruct *invalid = new FxHeaderStruct;
            invalid->name = "";
            invalid->fxID = "";
            invalid->fxVersion = 0;
            invalid->nParams = 0;

            emit validHeader(invalid);
        }
    }
}

//-----------------------------------------------------------------------------
void MainDialog::showCopyright()
{
    //show copyright information (GNU GPL)

    QMessageBox messageBox(this);
    messageBox.setWindowTitle(tr("About the VST Preset Generator "));

    QString text = tr("<h2>VST Preset Generator ");
    text += VpgVersion::getCurrentVersion().getStringVersionWithPlatform();
    text += tr("</h2>"
               "<p>Copyright &copy; 2007-2016 Fran&ccedil;ois Mazen"
               "<p>Please visit official website for more informations:"
               "<p><a href='http://vst-preset-generator.org/'>http://vst-preset-generator.org/</a>"
               "<p>This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version."
               "<p>This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details."
               "<p>You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."
               "<p>VST is a registered trademark of Steinberg Soft- und Hardware, GmbH.");
    text += tr("<p>Special thanks to: Selfik, AzralXt, Spacedad, and... YOU for using this tool!");
    text += tr("<p>English translation: ");
    text += tr("Translator");

    messageBox.setText(text);

    QImage logo(VPG_LOGO);
    QPixmap pixMap = QPixmap::fromImage(logo);
    if (!pixMap.isNull())
    {
        messageBox.setIconPixmap(pixMap);
    }
    messageBox.addButton(QMessageBox::Ok);
    messageBox.exec();
}

//-----------------------------------------------------------------------------
void MainDialog::closeEvent(QCloseEvent* event)
{
    bool mustQuitApplication = true;
    if(vstListModel->areDataSaved() == false && vstListModel->isListEmpty() == false)
    {
        QMessageBox::StandardButton buttonClicked = QMessageBox::question(this,
                                                                          tr("Closing..."),
                                                                          tr("Do you want to save changes you made to your VST list?"),
                                                                          QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);

        if(buttonClicked == QMessageBox::Save)
        {
            saveVstList();
        }
        else if(buttonClicked == QMessageBox::Cancel)
        {
            //quit the function, but not the application
            event->ignore();
            mustQuitApplication = false;
        }
    }

    if(mustQuitApplication)
    {
        //quit the application
        event->accept();
    }
}

//-----------------------------------------------------------------------------
void MainDialog::importParamFromVstOrPreset()
{
    // Create file filters.
    QString windowsFilesFilter(tr("VST File (*.dll *.fxp *.fxb)"));
    QString linuxFilesFilter(tr("Linux VST File (*.so)"));
    QString allFilesFilter(tr("Any File (*.*)"));

    QString filesFilter = windowsFilesFilter + ";;" + linuxFilesFilter + ";;" + allFilesFilter;

    // Select the default filter depending on the operating system.
    QString selectedFilter = windowsFilesFilter;
    if(QSysInfo::windowsVersion() == QSysInfo::WV_None)
    {
        selectedFilter = linuxFilesFilter;
    }

    // Get the name of the file
    QString fileName = QFileDialog::getOpenFileName(this,
                                                    tr("Load a Plugin VST"),
                                                    this->vpgSettings.getVSTPluginsFolder(),
                                                    filesFilter,
                                                    &selectedFilter);

    if(fileName != NULL)
    {
        if(fileName.endsWith (QString("dll")) || fileName.endsWith (QString("so")))
        {
            loadParamFromVst(fileName);
        }
        else
        {
            openPreset(fileName);
        }
    }
}

//-----------------------------------------------------------------------------
void MainDialog::loadParamFromVst(QString fileName)
{
    if(loadVst(fileName))
    {
        FxHeaderStruct *t_fxheaderstruct = new FxHeaderStruct;

        t_fxheaderstruct->fxID = EndiannessConverter::getMultiCharFromBigEndian(vpgVstHost->getFxId());
        t_fxheaderstruct->fxVersion = vpgVstHost->getFxVersion();
        t_fxheaderstruct->nParams = vpgVstHost->getNumberParams();
        t_fxheaderstruct->name = vpgVstHost->getName();

        t_fxheaderstruct->useChunk = vpgVstHost->useChunk();
        if(t_fxheaderstruct->useChunk)
        {
            t_fxheaderstruct->fxPath = fileName;
        }

        //add the new VST to our list:
        vstListModel->addPluginHeader(t_fxheaderstruct); //Maybe not ?

        //send parmeters to the headers object
        emit validHeader(t_fxheaderstruct);
    }
}


//-----------------------------------------------------------------------------
bool MainDialog::loadVst(QString fileName)
{
    bool isLoadingSuccessful = true;
    if(fileName != NULL)
    {
        QFileInfo info(fileName);
        this->vpgSettings.setVSTPluginsFolder(info.absolutePath());

        vpgVstHost->close(); //Unload before loading
        VpgVstHost::ERROR_CODE code = vpgVstHost->load(fileName);
        if(code != VpgVstHost::ERROR_NO_ERROR)
        {
            //error while opening
            vpgVstHost->close();
            QString message;
            message = tr("Error: ");
            switch(code)
            {
            case VpgVstHost::ERROR_PLUGIN_MAIN:
                message += tr("VST Plugin main entry not found");
                break;

            case VpgVstHost::ERROR_PLUGIN_INSTANCE:
                message += tr("failed to create effect instance");
                break;

            case VpgVstHost::ERROR_PLUGIN_VALID:
                message += tr("it's not a valid VST plugin");
                break;

            case VpgVstHost::ERROR_PLUGIN_LOAD:
            default:
                message += tr("failed to load VST Plugin library");
                message += tr("\n");
                message += fileName;
                break;
            }

            QMessageBox::critical(this, tr("Loading plugin VST"), message);
            isLoadingSuccessful = false;
        }
    }

    return isLoadingSuccessful;
}

//-----------------------------------------------------------------------------
void MainDialog::setGlobalRandomMethod(Parameter::RandomMode method)
{
    QMessageBox::StandardButton buttonClicked = QMessageBox::question(this,
                                                                      tr("Setting a random method"),
                                                                      tr("If you change the random method of all parameters, your old settings will be lost.\nDo you want to continue?"),
                                                                      QMessageBox::Yes | QMessageBox::No);

    if(buttonClicked == QMessageBox::Yes)
    {
        parameterTableModel->ChangeRandomMethodForAll(method);
        parameterTable->resizeColumnsToContents ();
    }
}

//-----------------------------------------------------------------------------
void MainDialog::setGlobalRandomMethodConstant()
{
    setGlobalRandomMethod(Parameter::RandomMode_Constant);
}

//-----------------------------------------------------------------------------
void MainDialog::setGlobalRandomMethodNormal()
{
    setGlobalRandomMethod(Parameter::RandomMode_Normal);
}

//-----------------------------------------------------------------------------
void MainDialog::setGlobalRandomMethodTotal()
{
    setGlobalRandomMethod(Parameter::RandomMode_Random);
}

//-----------------------------------------------------------------------------
void MainDialog::onVstListDoubleClicked(const QModelIndex &index)
{
    if(index.isValid())
    {
        FxHeaderStruct* fxHeader = vstListModel->getPluginHeaderAtIndex(index.row());
        if (fxHeader != NULL)
        {
            emit validHeader(fxHeader);
        }
    }
}

//-----------------------------------------------------------------------------
void MainDialog::setGlobalRandomMethodUniform()
{
    setGlobalRandomMethod(Parameter::RandomMode_Uniform);
}

