/*
-------------------------------------------------------------------------------
    This file is a part of the VST Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: vstpreset.h
  Author: Francois Mazen
  Date: 18/07/16 19:54
  Description: Class for VST Preset writer.
-------------------------------------------------------------------------------
*/


#ifndef VST_PRESET_H
#define VST_PRESET_H

class QFile;
class QString;

class FxHeader;
class ParameterTableModel;
class VpgVstHost;

class VSTPreset
{      
public:
    enum ERROR_CODE
    {
        ERROR_NO_ERROR,
        ERROR_COULD_NOT_CREATE_FILE,
        ERROR_VST_HOST_INVALID_PROGRAM_NUMBER
    };

    static ERROR_CODE WriteProgramToFile(const QString& filename,
                                         const FxHeader& fxHeader,
                                         const ParameterTableModel& parameterTableModel);
    static ERROR_CODE WriteBankToFile(const QString& filename,
                                      const FxHeader& fxHeader,
                                      const ParameterTableModel& parameterTableModel,
                                      int programCountPerBank);
    static ERROR_CODE WriteProgramChunkToFile(const QString& filename,
                                              const FxHeader& fxHeader,
                                              const ParameterTableModel& parameterTableModel,
                                              VpgVstHost& vstHost);
    static ERROR_CODE WriteBankChunkToFile(const QString& filename,
                                           const FxHeader& fxHeader,
                                           const ParameterTableModel& parameterTableModel,
                                           VpgVstHost& vstHost,
                                           int programCountPerBank);

private:
    static ERROR_CODE CreateQFile(const QString& filename, QFile& file);
    static ERROR_CODE WriteRawProgram(QFile& file,
                                      const FxHeader& fxHeader,
                                      const ParameterTableModel& parameterTableModel,
                                      int programNumber);
};

#endif //VST_PRESET_H
